(require (file "../spur.scm"))

;;;  This generates a simple bass and drum groove in the style of Uzeb.
;;;  This is a loop but using randomization to produce variations.
;;;  The GM MIDI notes (channel 10) are used for snare, hihat, and bass drum.
;;;  Channel 1 should be assigned to an electric bass.

(define snare 38)                  ; gm midi drum note
(define hihat 42)                  ; gm midi drum note
(define bdrum 36)                  ; gm midi drum note
(define bass-pgm 33)               ; gm midi program

(define met (make-metro 140))      ; tempo 
(define root 36)                   ; key to play in

;; this schedules each drum event
(define (perc beat pitch vel)
  (schedule-note (met beat) 10 pitch vel (* 30 *milli*)))

;; this schedules each bass event
(define (bass beat pitch vel dur)
  (schedule-note (met beat) 1 (+ pitch root) vel (met 'dur dur)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  The four musical loops

(define (bass-loop beat)
  (bass beat -2  (rand 91 101) 1)
  (bass (+ beat 1 -1/20) 0  78 4/20)
  (bass (+ beat 3 1/20) -2  72 13/20)
  (if (= (remainder beat 32) 24) (bass (+ beat 3 13/20) 3 91 10/20)
      (bass (+ beat 3 13/20) 0 105 5/20))
  (schedule (met (+ beat 5)) bass-loop (+ beat 8)))

(define (hihat-loop beat)
  (perc (+ beat 2/3 1/32) hihat 
    (if (= (remainder beat 16) 6) (rand 72 84) (rand 25 30)))
  (perc (+ beat 3/3) hihat (rand 50 80))
  (schedule (met (+ beat .9)) hihat-loop (+ beat 1)))

(define (bdrum-loop beat)
  (perc beat bdrum (rand 110 120))
  (perc (+ beat 3 2/3) bdrum (rand 100 115))
  (when (> (rand) .8) (perc (+ beat 2 2/3) bdrum (rand 70 100)))
  (when (> (rand) .4) (perc (+ beat 3) bdrum (rand 80 90)))
  (schedule (met (+ beat 6)) bdrum-loop (+ beat 8)))

(define (snare-loop beat) 
  (perc (+ beat 2) snare (rand 101 110))
  (perc (+ beat 6) snare (rand 101 125))
  (when (= (remainder beat 32) 0)
    (perc (+ beat 6 2/3 1/32) snare (rand 40 70))
    (perc (+ beat 7 2/3 1/32) snare (rand 70 90))
    (when (> (rand) .5)
      (perc (+ beat 7 0/3 2/32) snare (rand 10 30))
      (perc (+ beat 7 1/3 2/32) snare (rand 40 50)))) 
  (when (> (rand) .6) (perc (+ beat 1 2/3 1/32) snare (rand 30 50)))
  (when (> (rand) .6) (perc (+ beat 5 2/3 1/32) snare (rand 30 70)))
  (when (> (rand) .3) (perc (+ beat 4 2/3 1/32) snare (rand 75 85)))
  (schedule (met (+ beat 7.9)) snare-loop (+ beat 8)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  Main

(define (main) 
  (with-midi-schedule 
    ; (click-loop (met 'get-beat 1))
    (hihat-loop (met 'get-beat 8))
    (bdrum-loop (met 'get-beat 8))
    (snare-loop (met 'get-beat 8))
    (schedule (+ (now) (* 100 *milli*)) program-change 1 bass-pgm)
    (bass-loop (met 'get-beat 8)) ))

;; click track useful for debugging grooves
(define (click-loop beat)
  (perc beat 76 35)   ; woodblock
  (schedule (met (+ beat .8)) click-loop (+ beat 1)))
